'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; /**
                                                                                                                                                                                                                                                                   * Copyright (c) 2018-present The Palmer Group
                                                                                                                                                                                                                                                                   *
                                                                                                                                                                                                                                                                   * This source code is licensed under the MIT license found in the
                                                                                                                                                                                                                                                                   * LICENSE file in the root directory of this source tree.
                                                                                                                                                                                                                                                                   */

exports.matchImageSnapshotCommand = matchImageSnapshotCommand;
exports.addMatchImageSnapshotCommand = addMatchImageSnapshotCommand;

var _lodash = require('lodash.kebabcase');

var _lodash2 = _interopRequireDefault(_lodash);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const screenshotsFolder = Cypress.config('screenshotsFolder');
const fileServerFolder = Cypress.config('fileServerFolder');
const updateSnapshots = Cypress.env('updateSnapshots') || false;

const defaultFileNameTransform = (name = '', test) => (0, _lodash2.default)(`${test.title}-${name}`);

function matchImageSnapshotCommand(defaultOptions) {
  return function matchImageSnapshot(subject, name, commandOptions) {
    const options = _extends({}, defaultOptions, (typeof name === 'string' ? commandOptions : name) || {});

    const { fileNameTransform = defaultFileNameTransform } = options;
    const fileName = fileNameTransform(name, this.test);
    cy.task('Matching image snapshot', {
      screenshotsFolder,
      fileServerFolder,
      updateSnapshots,
      options
    });

    const target = subject ? subject : cy;
    target.screenshot(fileName, options);

    return cy.task('Recording snapshot results').then(({
      pass,
      added,
      updated,
      diffRatio,
      diffPixelCount,
      diffOutputPath
    }) => {
      if (!pass && !added && !updated) {
        const differencePercentage = diffRatio * 100;
        throw new Error(`Screenshot was ${differencePercentage}% different from saved snapshot with ${diffPixelCount} different pixels.\n  See diff for details: ${diffOutputPath}`);
      }
    });
  };
}

function addMatchImageSnapshotCommand(maybeName = 'matchImageSnapshot', maybeOptions) {
  const options = typeof maybeName === 'string' ? maybeOptions : maybeName;
  const name = typeof maybeName === 'string' ? maybeName : 'matchImageSnapshot';
  Cypress.Commands.add(name, matchImageSnapshotCommand(options), {
    prevSubject: 'optional'
  });
}